#include <iostream>
#include <fstream>
#include <iomanip>
#include "channelflow/flowfield.h"
#include "channelflow/dns.h"
#include "channelflow/utilfuncs.h"

using namespace std;
using namespace channelflow;

int main(int argc, char* argv[]) {

  string purpose("create a linear combination of flowfields\n\n"
		 "usage : addfouriermodes.x mode1 mode2 ... <inputfield> <outfield>\n"
		 "where u is a flowfield and mode1, are Fourier modes obtained from getfouriermode\n");
  ArgList args(argc, argv, purpose);


  if (argc % 2 != 0 && argc < 4) {
    cerr << purpose << endl;
    cerr << "please use at least one real/field pair and an output file as command-line arguments\n";
    exit(1);
  }

  FlowField u(uinname);

  // Example command line: argv0 mode0 mode uin uout
  // args.getstr n value:          4     3   2   1

  for (int n=3; n<argc-3; ++n) {
    string modename = args.getstr(n, "<basisfunc>", "fourier mode");
    Real c       = args.getreal(n+1, "<Real>", "coefficient");
    FlowField u(uname);
    u *= c;

    if (n==2) {
      cout << outname << "  = " << c << " * " << uname << endl;
      sum = u;
    }
    else {
      if (!sum.congruent(u)) {
	cerr << "FlowField " << uname << " is not compatible with previous fields.\n";
	exit(1);
      }
      cout << outname << " += " << c << " * " << uname << endl;
      sum += u;
    }
  }

  sum.save(outname);
  args.save("./");
}
