#include <iostream>
#include <fstream>
#include <iomanip>
#include "channelflow/flowfield.h"
#include "channelflow/dns.h"
#include "channelflow/utilfuncs.h"

using namespace std;
using namespace channelflow;

int main(int argc, char* argv[]) {

  string purpose("chop off the end of a box in Lz (assuming the subbox is nearly periodic)");
  ArgList args(argc, argv, purpose);  // Assign parameters

  const int Nznew     = args.getint("-Nz", "--Nz", 0, "new # z gridpoints [default==no change]");
  const bool padded   = args.getbool("-p", "--padded", true, "set padding modes to zero");
  const bool fixdiv   = args.getbool("-dv", "--divergence", true, "fix divergence and Dirichlet BCs");

  const string u0name = args.getstr(2, "<infield>",  "input flow field");
  const string u1name = args.getstr(1, "<outfield>", "output flow field");

  args.check();
  args.save("./");

  FlowField u0(u0name);
  u0.makePhysical();

  const int Nx = u0.Nx();
  const int Ny = u0.Ny();
  const int Nz = u0.Nz();
  const int Nd = u0.Nd();
  const Real a = u0.a();
  const Real b = u0.b();
  const Real Lx = u0.Lx();
  const Real Lz = u0.Lz();
  
  const Real Lznew = (Lz/Nz)*Nznew;
  

  FlowField u1(Nx,Ny,Nznew,Nd,Lx,Lznew,a,b, Physical, Physical);
  for (int i=0; i<Nd; ++i) 
    for (int ny=0; ny<Ny; ++ny) 
      for (int nx=0; nx<Nx; ++nx) 
	for (int nz=0; nz<Nznew; ++nz) 
	  u1(nx,ny,nz,i) = u0(nx,ny,nz,i);

  u0.makeSpectral();
  u1.makeSpectral();

  cout << setprecision(16);
  cout << "L2Norm(u0)  == " << L2Norm(u0) << endl;
  cout << "L2Norm(u1)  == " << L2Norm(u1) << endl;
  cout << "bcNorm(u0)  == " << bcNorm(u0) << endl;
  cout << "bcNorm(u1)  == " << bcNorm(u1) << endl;
  
  
  if (u0.Nd() == 3) {
    cout << "divNorm(u0) == " << divNorm(u0) << endl;
    cout << "divNorm(u1) == " << divNorm(u1) << endl;

    if (fixdiv) {
      cout <<"fixing divergence..." << endl;
      Vector v;
      FlowField foo(Nx,Ny,Nznew,Nd,Lx,Lznew,a,b);
      field2vector(u1,v);
      vector2field(v,foo);
      cout << "L2Dist(u1,2)== " << L2Dist(u1,foo) << endl;
      u1 = foo;
      cout << "L2Norm(u2)  == " << L2Norm(u1) << endl;
      cout << "divNorm(u2) == " << divNorm(u1) << endl;
      cout << "bcNorm(u2)  == " << bcNorm(u1) << endl;
    }
  }

  if (padded)
    u1.zeroPaddedModes();

  u1.save(u1name);
}
