#include <iostream>
#include <fstream>
#include <iomanip>
#include "channelflow/flowfield.h"
#include "channelflow/dns.h"
#include "channelflow/utilfuncs.h"

using namespace std;
using namespace channelflow;

int main(int argc, char* argv[]) {

  string purpose("zero all but the kx,kz == 0,0, 0,1 and 1,1 Fourier modes of a field");
  ArgList args(argc, argv, purpose);

  const int mymax = args.getint("-My", "--My", 999, "truncate my >= My Chebyshev modes");
  const string inname = args.getstr (2, "<field>", "input field");
  const string otname = args.getstr (1, "<field>", "output field");

  args.check();
  args.save("./");

  FlowField u(inname);
  u.makeSpectral();

  Complex zero(0.0, 0.0);

  for (int i=0; i<u.Nd(); ++i) {
    for (int mx=0; mx<u.Mx(); ++mx) {
      const int akx = abs(u.kx(mx));
      for (int mz=0; mz<u.Mz(); ++mz) {
	const int akz = abs(u.kz(mz));
	
	if (akx<=2 && akz<=2) {
	  for (int my=mymax; my<u.My(); ++my)
	    u.cmplx(mx,my,mz,i) = zero;
	}
	else {
	  for (int my=0; my<u.My(); ++my)
	    u.cmplx(mx,my,mz,i) = zero;
	}
      }
    }
  }
  u.save(otname);
} 

