#include <iostream>
#include <fstream>
#include <iomanip>
#include "channelflow/flowfield.h"
#include "channelflow/dns.h"
#include "channelflow/utilfuncs.h"

using namespace std;
using namespace channelflow;

void round(Complex& z) {
  Real a = real(z);
  Real b = imag(z);
  if (abs(a) < 1e-10)
    a = 0.0;
  if (abs(b) < 1e-10)
    b = 0.0;
  z = Complex(a,b);
}

int main(int argc, char* argv[]) {

  string purpose("print out all spectral coeffs greater than eps");
  ArgList args(argc, argv, purpose);

  const Real eps = args.getreal("-eps", "--epsilon", 1e-02, "print spectral coeffs > eps");
  const string uinname = args.getstr (2, "<field>", "input field");
  const string uotname = args.getstr (1, "<field>", "output field");

  args.check();
  args.save("./");

  FlowField u(uinname);
  u.makeSpectral();

  FlowField uout(u);
  uout.setToZero();
  
  Complex z(0.1, 0.0);
  int N = 0;
  cout << "norm(z) == " << norm(z) << endl;
  char c = ',';
  for (int i=0; i<u.Nd(); ++i) {
    for (int mx=0; mx<u.Mx(); ++mx) {
      for (int mz=0; mz<u.Mz(); ++mz) {
	for (int my=0; my<u.My(); ++my) {
	  Complex a = u.cmplx(mx,my,mz,i);
	  if (abs(a) > eps) {
	    round(a); 
	    cout << "u.cmplx(" 
		 << mx << c 
		 << my << c 
		 << mz << c 
		 << i << ") = Complex" << a << ";\n";
	    uout.cmplx(mx,my,mz,i) = a;
	    ++N;
	  }
	}
      }
    }
  }
  uout.save(uotname);
  cout << N << " large modes " << endl;
} 

